var fs = require('fs');
var Q = require('q');
var path = require('path');
var cheerio = require('cheerio');
var mkdirp = require('mkdirp');
var nunjucks = require('nunjucks');
var decamelize = require('decamelize');

var env = new nunjucks.Environment();
env.addExtension('CodeEditExtension', new CodeEditExtension());

var slidePath;
var slideRootPath;
var currentBody;
var slideContainer;

module.exports = {
    blocks: {
        revealjs: {
            process: function (block) {
        /*        if (this.generator !== 'website') {
                    return '';
                }

                var config = block.kwargs;

                if (!slideContainer[config.slideName]){
                    slideContainer[config.slideName] = '';
                }

                slideContainer[config.slideName] += currentBody;*/
                return '';
            }
        }
    },

    hooks: {
        "init": function () {
            slidePath = this.config.values.pluginsConfig.revealjs.slidePath || 'Slides';
            slideRootPath = this.config.values.pluginsConfig.revealjs.slideRootPath || '/Slides';
            slideContainer = {};
        },

        //after all pages have been loaded, create the appropriate slide pages
        "finish": function (page, rats) {
            var framePath = slidePath + '/frame.html';
            framePath = this.output.resolve(framePath);

            for (var key in slideContainer){
                createFile(key, framePath, this);
            }

            function foo() {
                var defer = Q.defer();
                setTimeout(function () {
                    console.log("C")
                    defer.resolve();
                }, 5000);
                return defer.promise;
            }

            return foo().then(function () {
                console.log('foo has been resolved!');
                return;
            });
        },

        "page:before": function (page, config) {
            var $ = cheerio.load('<html>' + page.content + '</html>');
            var body = '';
            var book = this;
            var sectionCount = 0;

            $('section').each(function (i, section) {
                var newData;
                var render = $(this).data('render') || 'both';

                if (render === 'book') {
                    return;
                }

                sectionCount++;
                if ($(this).html().indexOf('codeeditor') >= 0) {
                    var sectionBlock = newData = $(this).text();
                    sectionBlock = processBlock(sectionBlock);
                    $(this).html(sectionBlock);
                }
                else {
                    newData = $(this).text();
                }

                body += ($('<div>').append($(this).clone()).html());

                if (render === 'slide') {
                    $(this).remove();
                }
                else {
                    $(this).replaceWith(newData);
                }
            });

            if (body === '') {
                return page;
            }
            else {
                var slidePath = page.path.substring(4, page.path.lastIndexOf('/'));
                if (sectionCount > 1) {
                    currentBody = '<section>' + body + '</section>';
                }
                else{
                    currentBody = body;
                }

                if (!slideContainer[slidePath]){
                    slideContainer[slidePath] = '';
                }

                slideContainer[slidePath] += currentBody;

                var pageData = $('html').html();
                pageData = pageData.replace(/&apos;/g, "'").replace(/&quot;/g, '"');
                page.content = pageData;

                return page;
            }
        }
    }

};

function createNextLink(page) {
    if (!page.next) {
        return '';
    }

    var nextPagePath = slideRootPath + page.next.path.slice(4, -2) + 'html'
    var endSection = '<section><a href=' + nextPagePath + '>' + page.next.title + '</a></section>';
    return endSection;
}

function createFile(key, framePath, book){
    var revealData = '';
    var revealPath = slidePath + key + '/index.html';
    var body = slideContainer[key];
    var slideName = key.substring(key.lastIndexOf('/')+1);

    revealPath = book.output.resolve(revealPath);

    Q.nfcall(fs.readFile, framePath, "utf-8")
        .then(function (data) {
            console.log('a')
            revealData = data.replace(/\%body\%/g, body).replace(/\%root\%/g, slideRootPath).replace(/\%name\%/g, slideName);
            return Q.nfcall(mkdirp, path.dirname(revealPath));
        })
        .then(function () {
            console.log('b')
            return Q.nfcall(fs.writeFile, revealPath, revealData, 'utf-8');
        })
        .fail(function (error) {
            console.log(error);
        });
}

function processBlock(data) {
    return env.renderString(data, {});
}

function CodeEditExtension() {
    this.tags = ['codeeditor'];

    this.parse = function (parser, nodes, lexer) {
        // get the tag token
        var tok = parser.nextToken();

        // parse the args and move after the block end. passing true
        // as the second arg is required if there are no parentheses
        var args = parser.parseSignature(null, true);
        parser.advanceAfterBlockEnd(tok.value);

        // parse the body and possibly the error block, which is optional
        var body = parser.parseUntilBlocks('error', 'endcodeeditor');
        var errorBody = null;

        if (parser.skipSymbol('error')) {
            parser.skip(lexer.TOKEN_BLOCK_END);
            errorBody = parser.parseUntilBlocks('endcodeeditor');
        }

        parser.advanceAfterBlockEnd();

        // See above for notes about CallExtension
        return new nodes.CallExtension(this, 'run', args, [body, errorBody]);
    };

    this.run = function (context, args, body, errorBody) {
        var params = '';
        var newkey;

        for (var key in args) {
            if (args.hasOwnProperty(key)) {
                if (key === 'src') {
                    newkey = 'codeurl';
                }
                else {
                    newkey = decamelize(key, '-');
                }

                if (key !== '__keywords') {
                    if (args[key] != '') {
                        params += 'data-' + newkey + '="' + args[key] + '" ';
                    }
                    else {
                        params += 'data-' + newkey + ' ';
                    }
                }
            }
        }

        var ret = new nunjucks.runtime.SafeString('<h3>Example</h3><iframe  data-code-editor width="100%" height=400 ' + params + '>' + body() + '</iframe>');
        return ret;
    };
}

